import 'dart:io';

import 'package:comunic/enums/post_kind.dart';
import 'package:comunic/lists/users_list.dart';
import 'package:comunic/models/post.dart';
import 'package:comunic/models/user.dart';
import 'package:comunic/ui/tiles/comment_tile.dart';
import 'package:comunic/ui/widgets/account_image_widget.dart';
import 'package:comunic/ui/widgets/network_image_widget.dart';
import 'package:comunic/utils/date_utils.dart';
import 'package:comunic/utils/files_utils.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:comunic/utils/ui_utils.dart';
import 'package:flutter/material.dart';

/// Single posts tile
///
/// @author Pierre HUBERT

/// User style
const TextStyle _userNameStyle = TextStyle(
    color: Color.fromRGBO(0x72, 0xaf, 0xd2, 1.0), //#72afd2
    fontWeight: FontWeight.w600,
    fontSize: 16.0);

class PostTile extends StatefulWidget {
  final Post post;
  final UsersList usersInfo;
  final void Function(Post) onTapLike;

  const PostTile(
      {Key key,
      @required this.post,
      @required this.usersInfo,
      @required this.onTapLike})
      : assert(post != null),
        assert(usersInfo != null),
        assert(onTapLike != null),
        super(key: key);

  @override
  State<StatefulWidget> createState() => _PostTileState();
}

class _PostTileState extends State<PostTile> {
  // Class members
  TextEditingController _controller = TextEditingController();
  File _commentImage;

  User get _user => widget.usersInfo.getUser(widget.post.userID);

  bool get _commentValid => _controller.text.toString().length > 3;

  bool get _hasImage => _commentImage != null;

  Widget _buildHeaderRow() {
    // Header row
    return Row(
      children: <Widget>[
        // User account image
        Padding(
          padding: const EdgeInsets.only(right: 8.0, left: 8.0),
          child: AccountImageWidget(user: _user),
        ),

        // Column with user name + post target
        Expanded(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              Text(
                _user.displayName,
                style: _userNameStyle,
              ),
              Text(diffTimeFromNowToStr(widget.post.timeSent)),
            ],
          ),
        ),

        PopupMenuButton(
          itemBuilder: (c) => [],
        )
      ],
    );
  }

  Widget _buildContentRow() {
    Widget postContent;
    switch (widget.post.kind) {
      case PostKind.IMAGE:
        postContent = _buildPostImage();
        break;

      default:
    }

    return Column(
      children: <Widget>[
        // Post "rich" content
        Container(child: postContent),

        // Post text
        Container(
            child: widget.post.hasContent ? Text(widget.post.content) : null),
      ],
    );
  }

  Widget _buildButtonsArea() {
    return Padding(
      padding:
          const EdgeInsets.only(top: 16.0, left: 8.0, right: 8.0, bottom: 16.0),
      child: Column(
        children: <Widget>[
          // Like button
          Center(
            child: InkWell(
              onTap: () => widget.onTapLike(widget.post),
              child: Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Padding(
                    padding: const EdgeInsets.only(left: 8.0, right: 8.0),
                    child: Icon(
                      Icons.thumb_up,
                      color: widget.post.userLikes ? Colors.blue : null,
                    ),
                  ),
                  Text(widget.post.likes < 2
                      ? tr("%num% like",
                          args: {"num": widget.post.likes.toString()})
                      : tr("%num% likes",
                          args: {"num": widget.post.likes.toString()}))
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 1.0,
      child: Column(
        children: <Widget>[
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: Column(
              children: <Widget>[
                _buildHeaderRow(),
                _buildContentRow(),
                _buildButtonsArea(),
              ],
            ),
          ),
          Container(
            child: widget.post.hasComments ? _buildComments() : null,
          ),
        ],
      ),
    );
  }

  Widget _buildPostImage() {
    return NetworkImageWidget(
      url: widget.post.fileURL,
      allowFullScreen: true,
      roundedEdges: false,
    );
  }

  /// Build the list of comments
  Widget _buildComments() {
    assert(widget.post.hasComments);

    final comments = List<Widget>.generate(
      widget.post.comments.length,
      (num) => CommentTile(
            comment: widget.post.comments[num],
            user: widget.usersInfo.getUser(widget.post.comments[num].userID),
          ),
    );

    // Add comments form
    comments.add(_buildCommentsForm());

    return Container(
      color: Colors.grey[300],
      child: Padding(
        padding: const EdgeInsets.only(top: 8.0, bottom: 8.0),
        child: Column(
          children: comments,
        ),
      ),
    );
  }

  /// Build comments form
  Widget _buildCommentsForm() {
    return Padding(
      padding: EdgeInsets.only(
          left: 8.0,
          right: 8.0,
          top: (widget.post.comments.length > 0 ? 8.0 : 0)),
      child: Row(
        children: <Widget>[
          // Comment input
          Expanded(
            child: TextField(
              controller: _controller,
              onChanged: (s) => setState(() {}),
              onSubmitted: _commentValid ? (s) => _submitComment() : null,
              decoration: InputDecoration(
                  hintText: tr("New comment..."),
                  hintStyle: TextStyle(color: Colors.grey, fontSize: 12),
                  fillColor: Colors.white,
                  filled: true,
                  border: OutlineInputBorder(
                      borderSide: BorderSide(width: 0.5, color: Colors.grey),
                      borderRadius: BorderRadius.only(
                        topLeft: Radius.circular(3),
                        bottomLeft: Radius.circular(3),
                      ),
                      gapPadding: 1),
                  contentPadding: EdgeInsets.only(
                    left: 10,
                    right: 10,
                    bottom: 5,
                    top: 5,
                  )),
            ),
          ),

          // Image button
          Container(
            width: 30,
            child: FlatButton(
              padding: EdgeInsets.only(),
              onPressed: _pickImage,
              child: Icon(
                Icons.image,
                color: _hasImage ? Colors.blue : Colors.grey,
              ),
            ),
          ),

          // Submit button
          Container(
            width: 40,
            child: FlatButton(
              padding: EdgeInsets.only(),
              onPressed:
                  _commentValid || _hasImage ? () => _submitComment() : null,
              child: Icon(
                Icons.send,
                color: _commentValid || _hasImage ? Colors.blue : Colors.grey,
              ),
            ),
          ),
        ],
      ),
    );
  }

  /// Pick an image
  Future<void> _pickImage() async {
    // Ask the user to confirm image removal if there is already one selected
    if (_hasImage) {
      if (await askUserConfirmation(
        context: context,
        title: tr("Remove selected image"),
        message: tr("Do you want to unselected currently selected image ?"),
      )) {
        setState(() {
          _commentImage = null;
        });
      }

      return;
    }

    // Pick a new image
    final newImage = await pickImage(context);
    setState(() {
      _commentImage = newImage;
    });
  }

  /// Submit comment entered by the user
  Future<bool> _submitComment() async {}
}
